@@ -19,6 +19,7 @@ use crate::packet::{
     UserAttribute,
     UserID,
 };
+use crate::packet::signature::subpacket::NotationDataFlags;
 use crate::cert::prelude::*;
 
 /// A builder for revocation certificates for OpenPGP certificates.
@@ -83,7 +84,6 @@ pub struct CertRevocationBuilder {
 }
 assert_send_and_sync!(CertRevocationBuilder);
 
-#[allow(clippy::new_without_default)]
 impl CertRevocationBuilder {
     /// Returns a new `CertRevocationBuilder`.
     ///
@@ -183,6 +183,84 @@ impl CertRevocationBuilder {
         })
     }
 
+    /// Adds a notation to the revocation certificate.
+    ///
+    /// Unlike the [`CertRevocationBuilder::set_notation`] method, this function
+    /// does not first remove any existing notation with the specified name.
+    ///
+    /// See [`SignatureBuilder::add_notation`] for further documentation.
+    ///
+    /// [`SignatureBuilder::add_notation`]: crate::packet::signature::SignatureBuilder::add_notation()
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::packet::signature::subpacket::NotationDataFlags;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let builder = CertRevocationBuilder::new().add_notation(
+    ///     "revocation-policy@example.org",
+    ///     "https://policy.example.org/cert-revocation-policy",
+    ///     NotationDataFlags::empty().set_human_readable(),
+    ///     false,
+    /// );
+    /// # Ok(())
+    /// # }
+    pub fn add_notation<N, V, F>(self, name: N, value: V, flags: F,
+                                 critical: bool)
+        -> Result<Self>
+    where
+        N: AsRef<str>,
+        V: AsRef<[u8]>,
+        F: Into<Option<NotationDataFlags>>,
+    {
+        Ok(Self {
+            builder: self.builder.add_notation(name, value, flags, critical)?
+        })
+    }
+
+    /// Sets a notation to the revocation certificate.
+    ///
+    /// Unlike the [`CertRevocationBuilder::add_notation`] method, this function
+    /// first removes any existing notation with the specified name.
+    ///
+    /// See [`SignatureBuilder::set_notation`] for further documentation.
+    ///
+    /// [`SignatureBuilder::set_notation`]: crate::packet::signature::SignatureBuilder::set_notation()
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::packet::signature::subpacket::NotationDataFlags;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let builder = CertRevocationBuilder::new().set_notation(
+    ///     "revocation-policy@example.org",
+    ///     "https://policy.example.org/cert-revocation-policy",
+    ///     NotationDataFlags::empty().set_human_readable(),
+    ///     false,
+    /// );
+    /// # Ok(())
+    /// # }
+    pub fn set_notation<N, V, F>(self, name: N, value: V, flags: F,
+                                 critical: bool)
+        -> Result<Self>
+    where
+        N: AsRef<str>,
+        V: AsRef<[u8]>,
+        F: Into<Option<NotationDataFlags>>,
+    {
+        Ok(Self {
+            builder: self.builder.set_notation(name, value, flags, critical)?
+        })
+    }
+
     /// Returns a signed revocation certificate.
     ///
     /// A revocation certificate is generated for `cert` and signed
@@ -334,7 +412,6 @@ pub struct SubkeyRevocationBuilder {
 }
 assert_send_and_sync!(SubkeyRevocationBuilder);
 
-#[allow(clippy::new_without_default)]
 impl SubkeyRevocationBuilder {
     /// Returns a new `SubkeyRevocationBuilder`.
     ///
@@ -419,6 +496,84 @@ impl SubkeyRevocationBuilder {
         })
     }
 
+    /// Adds a notation to the revocation certificate.
+    ///
+    /// Unlike the [`SubkeyRevocationBuilder::set_notation`] method, this function
+    /// does not first remove any existing notation with the specified name.
+    ///
+    /// See [`SignatureBuilder::add_notation`] for further documentation.
+    ///
+    /// [`SignatureBuilder::add_notation`]: crate::packet::signature::SignatureBuilder::add_notation()
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::packet::signature::subpacket::NotationDataFlags;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let builder = CertRevocationBuilder::new().add_notation(
+    ///     "revocation-policy@example.org",
+    ///     "https://policy.example.org/cert-revocation-policy",
+    ///     NotationDataFlags::empty().set_human_readable(),
+    ///     false,
+    /// );
+    /// # Ok(())
+    /// # }
+    pub fn add_notation<N, V, F>(self, name: N, value: V, flags: F,
+                                 critical: bool)
+        -> Result<Self>
+    where
+        N: AsRef<str>,
+        V: AsRef<[u8]>,
+        F: Into<Option<NotationDataFlags>>,
+    {
+        Ok(Self {
+            builder: self.builder.add_notation(name, value, flags, critical)?
+        })
+    }
+
+    /// Sets a notation to the revocation certificate.
+    ///
+    /// Unlike the [`SubkeyRevocationBuilder::add_notation`] method, this function
+    /// first removes any existing notation with the specified name.
+    ///
+    /// See [`SignatureBuilder::set_notation`] for further documentation.
+    ///
+    /// [`SignatureBuilder::set_notation`]: crate::packet::signature::SignatureBuilder::set_notation()
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::packet::signature::subpacket::NotationDataFlags;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let builder = CertRevocationBuilder::new().set_notation(
+    ///     "revocation-policy@example.org",
+    ///     "https://policy.example.org/cert-revocation-policy",
+    ///     NotationDataFlags::empty().set_human_readable(),
+    ///     false,
+    /// );
+    /// # Ok(())
+    /// # }
+    pub fn set_notation<N, V, F>(self, name: N, value: V, flags: F,
+                                 critical: bool)
+        -> Result<Self>
+    where
+        N: AsRef<str>,
+        V: AsRef<[u8]>,
+        F: Into<Option<NotationDataFlags>>,
+    {
+        Ok(Self {
+            builder: self.builder.set_notation(name, value, flags, critical)?
+        })
+    }
+
     /// Returns a signed revocation certificate.
     ///
     /// A revocation certificate is generated for `cert` and `key` and
@@ -581,7 +736,6 @@ pub struct UserIDRevocationBuilder {
 }
 assert_send_and_sync!(UserIDRevocationBuilder);
 
-#[allow(clippy::new_without_default)]
 impl UserIDRevocationBuilder {
     /// Returns a new `UserIDRevocationBuilder`.
     ///
@@ -673,6 +827,84 @@ impl UserIDRevocationBuilder {
         })
     }
 
+    /// Adds a notation to the revocation certificate.
+    ///
+    /// Unlike the [`UserIDRevocationBuilder::set_notation`] method, this function
+    /// does not first remove any existing notation with the specified name.
+    ///
+    /// See [`SignatureBuilder::add_notation`] for further documentation.
+    ///
+    /// [`SignatureBuilder::add_notation`]: crate::packet::signature::SignatureBuilder::add_notation()
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::packet::signature::subpacket::NotationDataFlags;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let builder = CertRevocationBuilder::new().add_notation(
+    ///     "revocation-policy@example.org",
+    ///     "https://policy.example.org/cert-revocation-policy",
+    ///     NotationDataFlags::empty().set_human_readable(),
+    ///     false,
+    /// );
+    /// # Ok(())
+    /// # }
+    pub fn add_notation<N, V, F>(self, name: N, value: V, flags: F,
+                                 critical: bool)
+        -> Result<Self>
+    where
+        N: AsRef<str>,
+        V: AsRef<[u8]>,
+        F: Into<Option<NotationDataFlags>>,
+    {
+        Ok(Self {
+            builder: self.builder.add_notation(name, value, flags, critical)?
+        })
+    }
+
+    /// Sets a notation to the revocation certificate.
+    ///
+    /// Unlike the [`UserIDRevocationBuilder::add_notation`] method, this function
+    /// first removes any existing notation with the specified name.
+    ///
+    /// See [`SignatureBuilder::set_notation`] for further documentation.
+    ///
+    /// [`SignatureBuilder::set_notation`]: crate::packet::signature::SignatureBuilder::set_notation()
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::packet::signature::subpacket::NotationDataFlags;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let builder = CertRevocationBuilder::new().set_notation(
+    ///     "revocation-policy@example.org",
+    ///     "https://policy.example.org/cert-revocation-policy",
+    ///     NotationDataFlags::empty().set_human_readable(),
+    ///     false,
+    /// );
+    /// # Ok(())
+    /// # }
+    pub fn set_notation<N, V, F>(self, name: N, value: V, flags: F,
+                                 critical: bool)
+        -> Result<Self>
+    where
+        N: AsRef<str>,
+        V: AsRef<[u8]>,
+        F: Into<Option<NotationDataFlags>>,
+    {
+        Ok(Self {
+            builder: self.builder.set_notation(name, value, flags, critical)?
+        })
+    }
+
     /// Returns a signed revocation certificate.
     ///
     /// A revocation certificate is generated for `cert` and `userid`
@@ -842,7 +1074,6 @@ pub struct UserAttributeRevocationBuilder {
 }
 assert_send_and_sync!(UserAttributeRevocationBuilder);
 
-#[allow(clippy::new_without_default)]
 impl UserAttributeRevocationBuilder {
     /// Returns a new `UserAttributeRevocationBuilder`.
     ///
@@ -933,6 +1164,84 @@ impl UserAttributeRevocationBuilder {
         })
     }
 
+    /// Adds a notation to the revocation certificate.
+    ///
+    /// Unlike the [`UserAttributeRevocationBuilder::set_notation`] method, this function
+    /// does not first remove any existing notation with the specified name.
+    ///
+    /// See [`SignatureBuilder::add_notation`] for further documentation.
+    ///
+    /// [`SignatureBuilder::add_notation`]: crate::packet::signature::SignatureBuilder::add_notation()
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::packet::signature::subpacket::NotationDataFlags;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let builder = CertRevocationBuilder::new().add_notation(
+    ///     "revocation-policy@example.org",
+    ///     "https://policy.example.org/cert-revocation-policy",
+    ///     NotationDataFlags::empty().set_human_readable(),
+    ///     false,
+    /// );
+    /// # Ok(())
+    /// # }
+    pub fn add_notation<N, V, F>(self, name: N, value: V, flags: F,
+                                 critical: bool)
+        -> Result<Self>
+    where
+        N: AsRef<str>,
+        V: AsRef<[u8]>,
+        F: Into<Option<NotationDataFlags>>,
+    {
+        Ok(Self {
+            builder: self.builder.add_notation(name, value, flags, critical)?
+        })
+    }
+
+    /// Sets a notation to the revocation certificate.
+    ///
+    /// Unlike the [`UserAttributeRevocationBuilder::add_notation`] method, this function
+    /// first removes any existing notation with the specified name.
+    ///
+    /// See [`SignatureBuilder::set_notation`] for further documentation.
+    ///
+    /// [`SignatureBuilder::set_notation`]: crate::packet::signature::SignatureBuilder::set_notation()
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::packet::signature::subpacket::NotationDataFlags;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let builder = CertRevocationBuilder::new().set_notation(
+    ///     "revocation-policy@example.org",
+    ///     "https://policy.example.org/cert-revocation-policy",
+    ///     NotationDataFlags::empty().set_human_readable(),
+    ///     false,
+    /// );
+    /// # Ok(())
+    /// # }
+    pub fn set_notation<N, V, F>(self, name: N, value: V, flags: F,
+                                 critical: bool)
+        -> Result<Self>
+    where
+        N: AsRef<str>,
+        V: AsRef<[u8]>,
+        F: Into<Option<NotationDataFlags>>,
+    {
+        Ok(Self {
+            builder: self.builder.set_notation(name, value, flags, critical)?
+        })
+    }
+
     /// Returns a signed revocation certificate.
     ///
     /// A revocation certificate is generated for `cert` and `ua` and
